#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAIN_C
#include "exitfail.h"

#define BUF_SIZE 2048

static int terminated = 0;

#define BASENAME(p) ((strrchr((p), '/') ? : ((p) - 1)) +1)
#define ARRAY_SIZE(a) (sizeof(a) /sizeof((a)[0]))

#define __unused __attribute__((unused))

/**** sample: RN4020/IoT Smart response: begin ****/
static unsigned char res_notify[] = "Notification handle = 0x0012"; /* iot smart response: notify from custom1*/
/**** sample: RN4020/IoT Smart response: begin ****/

/**** sample: IoT Smart motion data: begin ****/
#define ISM_MOTION_SIZE 120

static struct {
	float mag_x;
	float mag_y;
	float mag_z;
	float acc_x;
	float acc_y;
	float acc_z;
	int time_ms;
	int time_s;
	int time_m;
	int time_h;
} ism_motion_data[ISM_MOTION_SIZE];
/**** sample: IoT Smart motion data: end ****/

static void terminate_sig_handler(__unused int signum)
{
	terminated = 1;
}

static void set_sig_handler(int sig_list[], ssize_t num, __sighandler_t handler)
{
	struct sigaction sa;
	int i;

	memset(&sa, 0, sizeof(sa));
	sa.sa_handler = handler;

	for(i=0; i<num; i++)
		if(sigaction(sig_list[i], &sa, NULL) < 0)
			exitfail_errno("sigaction");
}

/**** sample: char(HEX) to int: begin ****/ 
int ch2i(unsigned char c) {
	int ret;
	
	ret = 0;

	if(c>='0' && c<='9') ret = (unsigned int)c - (unsigned int)'0';	
	else if(c>='a' && c <='f') ret = (unsigned int)c - (unsigned int)'a' + 10;	
	else if(c>='A' && c <='F') ret = (unsigned int)c - (unsigned int)'A' + 10;
	else ret = -1;

	return ret;	
}
/**** sample: char(HEX) to int: end ****/ 

int main(int argc, char *argv[])
{
	int terminate_sig_list[] = {
		SIGHUP, SIGINT, SIGQUIT, SIGPIPE, SIGTERM
	};

/**** sample: variable: begin ****/
       char buf1[BUF_SIZE];
	char env_file[100];
	char motion_file[100];
	int i, j;
	FILE *fp;
	int cnt_cmp;
/**** sample: variable: end ****/

/**** sample: variable for senseor data: begin ****/
	float mag_x, mag_y, mag_z;
	float acc_x, acc_y, acc_z;
	int time_ms, time_s, time_m, time_h;
	float pres, hum, temp, uv, amb;
	int date_d, date_m, date_y;
	int cnt_motion;

	int cal_tmp;
/**** sample: for senseor data: end ****/

	exitfail_init();

	if(argc < 2) {
		printf("Usage: %s <id>\n", BASENAME(argv[0]));
		return EXIT_SUCCESS;
	}

	set_sig_handler(terminate_sig_list, ARRAY_SIZE(terminate_sig_list),
			terminate_sig_handler);

	for(i = 0; i < 100; i++) {
		env_file[i] = '\0';
		motion_file[i] = '\0';
	}
	sprintf(env_file, "/root/log_ism_env_%s.txt", argv[1]);
	sprintf(motion_file, "/root/log_ism_motion_%s.txt", argv[1]);

/**** sample: Analize BLE Notification : begin ****/
	cnt_motion = 0;

	while(!terminated) {
		gets(buf1);
		cnt_cmp = 0;
		for(i = 0; i < (int)sizeof(res_notify) - 1; i++) 
			if(buf1[i] == res_notify[i]) cnt_cmp++;
		if(cnt_cmp == sizeof(res_notify) - 1 && buf1[36] == 'f' && buf1[37] == '2') {
/**** <--- Indent Shift: begin ****/
/**** sample: analize sensor data(motion): begin ****/
cal_tmp = (((ch2i(buf1[45]) * 16 + ch2i(buf1[46])) * 16 + ch2i(buf1[42])) * 16 + ch2i(buf1[43]));
if((ch2i(buf1[45]) & 0x8) == 0x8) cal_tmp |= ~0xffff; 
mag_x = (float)cal_tmp * 0.15;

cal_tmp = (((ch2i(buf1[51]) * 16 + ch2i(buf1[52])) * 16 + ch2i(buf1[48])) * 16 + ch2i(buf1[49]));
if((ch2i(buf1[51]) & 0x8) == 0x8) cal_tmp |= ~0xffff; 
mag_y = (float)cal_tmp * 0.15;

cal_tmp = (((ch2i(buf1[57]) * 16 + ch2i(buf1[58])) * 16 + ch2i(buf1[54])) * 16 + ch2i(buf1[55]));
if((ch2i(buf1[57]) & 0x8) == 0x8) cal_tmp |= ~0xffff; 
mag_z = (float)cal_tmp * 0.15;

cal_tmp = (((ch2i(buf1[63]) * 16 + ch2i(buf1[64])) * 16 + ch2i(buf1[60])) * 16 + ch2i(buf1[61]));
if((ch2i(buf1[63]) & 0x8) == 0x8) cal_tmp |= ~0xffff; 
acc_x = (float)cal_tmp / 4096; /* 4096: In case of 2G Range */

cal_tmp = (((ch2i(buf1[69]) * 16 + ch2i(buf1[70])) * 16 + ch2i(buf1[66])) * 16 + ch2i(buf1[67]));
if((ch2i(buf1[69]) & 0x8) == 0x8) cal_tmp |= ~0xffff; 
acc_y = (float)cal_tmp / 4096; /* 4096: In case of 2G Range */

cal_tmp = (((ch2i(buf1[75]) * 16 + ch2i(buf1[76])) * 16 + ch2i(buf1[72])) * 16 + ch2i(buf1[73]));
if((ch2i(buf1[75]) & 0x8) == 0x8) cal_tmp |= ~0xffff; 
acc_z = (float)cal_tmp / 4096; /* 4096: In case of 2G Range */

time_ms = (float)(((ch2i(buf1[81]) * 16 + ch2i(buf1[82])) * 16 + ch2i(buf1[78])) * 16 + ch2i(buf1[79]));
time_s = ch2i(buf1[84]) * 16 + ch2i(buf1[85]);
time_m = ch2i(buf1[87]) * 16 + ch2i(buf1[88]);
time_h = ch2i(buf1[90]) * 16 + ch2i(buf1[91]);
/**** sample: analize sensor data(motion): end ****/

/**** sample: sensor data(motion) to buffer: begin ****/
ism_motion_data[cnt_motion].mag_x = mag_x;
ism_motion_data[cnt_motion].mag_y = mag_y;
ism_motion_data[cnt_motion].mag_z = mag_z;
ism_motion_data[cnt_motion].acc_x = acc_x;
ism_motion_data[cnt_motion].acc_y = acc_y;
ism_motion_data[cnt_motion].acc_z = acc_z;
ism_motion_data[cnt_motion].time_ms = time_ms;
ism_motion_data[cnt_motion].time_s = time_s;
ism_motion_data[cnt_motion].time_m = time_m;
ism_motion_data[cnt_motion].time_h = time_h;
if(cnt_motion < 100) cnt_motion++;
/**** sample: sensor data(motion) to buffer: end ****/
/**** <--- Indent Shift: end ****/
		}
		else if(cnt_cmp == sizeof(res_notify) - 1 && buf1[36] == 'f' && buf1[37] == '3') {
/**** <--- Indent Shift: begin ****/
/**** sample: analize sensor data(env): begin ****/
pres = (float)(((ch2i(buf1[45]) * 16 + ch2i(buf1[46])) * 16 + ch2i(buf1[42])) * 16 + ch2i(buf1[43])) * 860 / 65535 + 250;
hum = (float)((((ch2i(buf1[51]) * 16 + ch2i(buf1[52])) * 16 + ch2i(buf1[48])) * 16 + ch2i(buf1[49])) - 896) / 64;
temp = (float)((((ch2i(buf1[57]) * 16 + ch2i(buf1[58])) * 16 + ch2i(buf1[54])) * 16 + ch2i(buf1[55])) - 2096) / 50;
uv = (float)(((ch2i(buf1[63]) * 16 + ch2i(buf1[64])) * 16 + ch2i(buf1[60])) * 16 + ch2i(buf1[61])) / (100 * 0.388);
amb = (float)(((ch2i(buf1[69]) * 16 + ch2i(buf1[70])) * 16 + ch2i(buf1[66])) * 16 + ch2i(buf1[67]));
if(uv < 0.814)
  amb = amb / (0.05 * 0.928 * (0.3102 * uv + 0.8525));
else
  amb = amb / (0.05 * 0.928 * (0.03683 * uv + 1.075));
date_d = ch2i(buf1[84]) * 16 + ch2i(buf1[85]); 
date_m = ch2i(buf1[87]) * 16 + ch2i(buf1[88]); 
date_y = ch2i(buf1[90]) * 16 + ch2i(buf1[91]); 
/**** sample: analize sensor data(env): end ****/

/**** output sensor data(env) to stdout: begin ****/
printf("\n****************************************************\n");
printf("\n---------[%s] sensor data(env) --------\n", argv[1]);
printf("%s\n", buf1);
if(buf1[45] == '0' && buf1[46] == '0' && buf1[42] == '0' && buf1[43] == '0')
	printf("Pressure[hPa]: not selected\n");
else 
	printf("Pressure[hPa]: %.2f\n", pres);

if(buf1[51] == '8' && buf1[52] == '0' && buf1[48] == '0' && buf1[49] == '0')
	printf("Humidity[%%RH]: not selected\n");
else 
	printf("Humidity[%%RH]: %.2f\n", hum);

if(buf1[57] == '8' && buf1[58] == '0' && buf1[54] == '0' && buf1[55] == '0')
	printf("Temparature[degC]: not selected\n");
else 
	printf("Temparature[degC]: %.2f\n", temp);

if(buf1[63] == '8' && buf1[64] == '0' && buf1[60] == '0' && buf1[61] == '0')
	printf("UV[mW/cm^2]: not selected\n");
else 
	printf("UV[mW/cm^2]: %.2f\n", uv);

if(buf1[69] == '8' && buf1[70] == '0' && buf1[66] == '0' && buf1[67] == '0')
	printf("Ambient[Lx]: not selected\n");
else
	printf("Ambient[Lx]: %.2f\n", amb);

printf("Date: %d.%d.%d\n", date_y, date_m, date_d);
/**** output sensor data(env) to stdout: end ****/

/**** sample: sensor data(env) to log file for post to heroku: begin ****/
fp = fopen(env_file, "w");

if(buf1[45] == '0' && buf1[46] == '0' && buf1[42] == '0' && buf1[43] == '0')
	fprintf(fp, "pres=None&");
else 
	fprintf(fp, "pres=%.2f&", pres);

if(buf1[51] == '8' && buf1[52] == '0' && buf1[48] == '0' && buf1[49] == '0')
	fprintf(fp, "hum=None&");
else 
	fprintf(fp, "hum=%.2f&", hum);

if(buf1[57] == '8' && buf1[58] == '0' && buf1[54] == '0' && buf1[55] == '0')
	fprintf(fp, "tmp=None&");
else 
	fprintf(fp, "tmp=%.2f&", temp);

if(buf1[63] == '8' && buf1[64] == '0' && buf1[60] == '0' && buf1[61] == '0')
	fprintf(fp, "uv=None&");
else 
	fprintf(fp, "uvi=%.2f&", uv);

if(buf1[69] == '8' && buf1[70] == '0' && buf1[66] == '0' && buf1[67] == '0')
	fprintf(fp, "amb=None&");
else
	fprintf(fp, "lux=%.2f&", amb);
fprintf(fp, "date=%d.%d.%d\n", date_y, date_m, date_d);

fclose(fp);
/**** sample: sensor data(env) to log file for post to heroku: end ****/

/**** sample: output latest sensor data(motion) to stdout: end ****/
printf("\n---------[%s] sensor data(motion) --------\n", argv[1]);
printf("Magnetic[uT]: (x, y, z) = (%.2f, %.2f, %.2f)\n", 
	ism_motion_data[cnt_motion - 1].mag_x, 
	ism_motion_data[cnt_motion - 1].mag_y, 
	ism_motion_data[cnt_motion - 1].mag_z);
printf("Acceleration[G]: (x, y, z) = (%.2f, %.2f, %.2f)\n", 
	ism_motion_data[cnt_motion - 1].acc_x, 
	ism_motion_data[cnt_motion - 1].acc_y, 
	ism_motion_data[cnt_motion - 1].acc_z);
printf("Time Stamp: %d:%d:%d (%d[ms])\n", 
	ism_motion_data[cnt_motion - 1].time_h, 
	ism_motion_data[cnt_motion - 1].time_m, 
	ism_motion_data[cnt_motion - 1].time_s, 
	ism_motion_data[cnt_motion - 1].time_ms);
/**** sample: output latest sensor data(motion) to stdout: end ****/

/**** sample: sensor data(motion) to log file: begin ****/
fp = fopen(motion_file, "w");

for(j = 0; j < cnt_motion; j++)
	fprintf(fp, "time=%d:%d:%d(%d[ms])&mag=(%.2f, %.2f, %.2f)&acc=(%.2f, %.2f, %.2f)\n", 
		ism_motion_data[j].time_h, 
		ism_motion_data[j].time_m, 
		ism_motion_data[j].time_s, 
		ism_motion_data[j].time_ms, 
		ism_motion_data[j].mag_x, 
		ism_motion_data[j].mag_y, 
		ism_motion_data[j].mag_z, 
		ism_motion_data[j].acc_x, 
		ism_motion_data[j].acc_y, 
		ism_motion_data[j].acc_z);
fclose(fp);
cnt_motion = 0;
/**** sample: sensor data(motion) to log file: end ****/
/**** <--- Indent Shift: end ****/
		}
	}
/**** sample: Analize BLE Notification : end ****/
	return EXIT_SUCCESS;
}
